/*==========================================================================
    Ginzburg-Pitaevskii equation
      ∂t w=w-norm(w)*w+△w
===========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <sys/wait.h>

#include <XRS/Complex_Control.h>
#include <XRS/Graphic.h>
using namespace XRS;

#define  Random   drand48()
#define  Dispstep 100
#define  Color    12

/************** 大域変数と関数宣言 ********************/

Complex   **w;    // 場
double    LX,LY;  // システムサイズ
double    DX,DY;  // 空間メッシュサイズ
double    t;      // 時間
int       nw=-10; // 初期条件の波数
int       ncx=0,ncy=0; // w*exp(2πi*(ncx/LX,ncy/LY)*(x,y)) を表示。

void            randinitial(int);
void            expinitial(int);
void            boundary();


/*---------------- 以下 XWindow 用 --------------------*/

Complex_Control_Window CW;
Graphic_Window  W,SW;
int             Wimax,Wjmax;
int             Initial=0;
double          Amp=0.8;

unsigned long*  colordisk();
int             diskcolor(Complex w);
void            draw();
void            SW_draw();
void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_GP.pdf",NULL);}  }


int main(int argc,char **argv)
{
   double    DT;

   /*--------------- initialize ---------------------*/

   int ar=0;
   ar++; if(argc>ar) LX=atof(argv[ar]); else LX=200;
   ar++; if(argc>ar) LY=atof(argv[ar]); else LY=LX;
   ar++; if(argc>ar) DX=atof(argv[ar]); else DX=1.0;  DY=DX;
   ar++; if(argc>ar) DT=atof(argv[ar]); else DT=0.1;
   
   fprintf(stderr,"usage%% GP [LX] [LY] [DX] [DT]\n"
          " current systemsize: (LX,LY)=(%g,%g)\n"
	  " space and time meshsize: DX=%g DT=%g\n"
   	  ,LX,LY,DX,DT); 
   
   Wimax=(int)(LX/DX)+2;
   Wjmax=(int)(LY/DY)+2;
   array(w,Wimax,Wjmax);
   Complex   **w2;  array(w2,Wimax,Wjmax);   

   /*---------------- setup window -------------------*/

   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;
   
   CW<<t<<"t"<<randinitial<<"random"<<expinitial<<"stationary"
       <<PARAMETER
       << nw << ncx << ncy << Amp
       <<"nw"<<"ncx"<<"ncy"<<"Amp";
   
   W.geometry(Wimax,Wjmax,1)<<"Gintzburg Pitaevskii equation"
       <<COLOR(colordisk(),(2*Color+1)*(2*Color+1));

   SW.geometry(2*Color+1,2*Color+1,4)<<"color sample"
       <<COLOR(colordisk(),(2*Color+1)*(2*Color+1));

   initialize();
   randinitial(0);

   SW_draw();
   draw();
   
   /*--------------- main roution -------------------*/

   while(1){
      checkevent();
      Complex DTxx=DT/(DX*DX);

      for(int ts=0; ts<Dispstep;ts++){

         for(int i=1; i<Wimax-1; i++)
	    for(int j=1; j<Wjmax-1; j++){
	       Complex dwxx=w[i-1][j]+w[i+1][j]+w[i][j-1]+w[i][j+1]-4.0*w[i][j];
               w2[i][j]=w[i][j]+DTxx*dwxx;
	    }
	  
         for(int i=1; i<Wimax-1; i++)
	    for(int j=1; j<Wjmax-1; j++)
               w[i][j]=w2[i][j]
		      +DT*(1.0-norm(w[i][j]))*w[i][j];

	 boundary();
      }
      
      t+=DT*Dispstep;
      draw();
   }
}


/**********************  初期条件 *************************/

void randinitial(int)
{  
   for(int i=0; i<Wimax; i++)
      for(int j=0; j<Wjmax; j++)
          w[i][j]=polar((double)Random,(double)(2*M_PI*Random));

   boundary();
   t=0; Initial=1;
}


void expinitial(int)
{
   double x,q;
   for(int i=0; i<Wimax; i++)
      for(int j=0; j<Wjmax; j++){
	  x=nw*(double)(i-1)/(Wimax-2)+0.0000001*Random;
	  q=2*M_PI*nw/(DX*(Wimax-2));
	  w[i][j]=sqrt(1.0-q*q)*exp(Complex(0,2*M_PI*x));
      }	  

   boundary();
   t=0; Initial=1;
}


/**********************  境界条件 *************************/

void boundary()
{
// periodic boundary condition
    for(int j=0;j<Wjmax;j++){
        w[0][j]=w[Wimax-2][j];
        w[Wimax-1][j]=w[1][j];
    }
    
    for(int i=0;i<Wimax;i++){
        w[i][0]=w[i][Wjmax-2];
        w[i][Wjmax-1]=w[i][1];
    }
}    


/********************** 書く *************************/

void draw(void)
{
    double  kcx;
    Complex z,ii(0,2.0*M_PI);
    
    if(Initial){ W.clear(); Initial=0; }

    for(int i=0; i<Wimax; i++)
       for(int j=0; j<Wjmax; j++){
  	   kcx=ncx*(double)(i-1)/(Wimax-2)+ncy*(double)(j-1)/(Wjmax-2);	   
           z=w[i][j]*exp(ii*kcx);
  	   W.point(diskcolor(Amp*z),i,j);
       }
}


void SW_draw()
{
    for(int i=0; i<2*Color+1; i++)
       for(int j=0; j<2*Color+1; j++)
  	   SW.point(i+j*(2*Color+1),i,2*Color-j);
}


/********************** 色の配列 ***********************/

unsigned long* colordisk()
{
    unsigned long *cp=new unsigned long [(2*Color+1)*(2*Color+1)];
    double r,phi;
    int i,j,k;
    for(i=-Color; i<Color+1; i++)
        for(j=-Color; j<Color+1; j++){
            k=i+Color+(j+Color)*(2*Color+1);
            r=sqrt((double)(i*i+j*j))/(double)Color;
            if(i==0 && j==0) phi=0; 
            else             phi=atan2((double)i,(double)j)/(2*M_PI);
            cp[k]=color_pixel(phi,r,1);
        }
    return cp;
}    


/*------------------- 配列の番号を返す ----------------*/

int diskcolor(Complex w)
{
    static int cn=2*Color+1;
    w+=Complex(1,1); w*=Color;
    return (int)real(w)+(int)imag(w)*cn;
}

