/*=======================================================================

           Gray-Scott Model

     ∂t U = ∂xx U - UVV + A(1-U)
     ∂t V = δδ ∂xx V + UVV + BV
   
========================================================================*/  

/*********** include **************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <XRS/Control.h>
#include <XRS/Plane.h>
#include <XRS/Simple.h>
#include "include/parameter_file.h"
using namespace XRS;

/*********** define ***************/

#define PI          M_PI
#define integer(x)  ((int)(floor(x)+floor(2*((x)-floor(x)))))
#define RAN         (drand48()-0.5)
#define XMark(d,w,gc,x,y,r) XDrawArc(d,w,gc,x-r,y-r,2*r,2*r,0,23040);


/*********** global variable ************/

double              t,L;
int                 N;
double              *u,*v;

double              delta,A,B;
double              DT,DX,u_start,v_start;
int                 i_max,j_max,init=0,DISPSTEP=100;

Display             *d;
Plane_Window        win;
Simple_Window       SW;
Control_Window      CW;
GC                  black,white;
double              dispu=1,dispv=1;
int                 SWi,SWj;


/********** function & class prototype *****************/

void initial(int);
void draw(int);
void noise(int);
void perturb(int);


/********** main ***********/

int main(int argc,char **argv)
{
    char     parafile[1024],paraset[1024];
    int ar=0;
    strcpy(parafile,argv[ar]); strcat(parafile,".para");
    ar++; if(argc>ar) strcpy(paraset,argv[ar]); else strcpy(paraset,"default");
    ar++; if(argc>ar) strcpy(parafile,argv[ar]);

    parameter_file(argc,argv,parafile,paraset,
          "DISPSTEP:%d L:%lf DX:%lf DT:%lf delta:%lf A:%lf B:%lf",
          &DISPSTEP,  &L,   &DX,   &DT,   &delta,   &A,   &B     );

    N=integer(L/DX);
    i_max=N;   j_max=200;
    SWi=i_max; SWj=500;
    u_start=1e-1;
    v_start=1e-1;    

    u =new double [N+1];   double *u2=new double [N+1];
    v =new double [N+1];   double *v2=new double [N+1];

/*----- initialize ----------*/

    CW<< initial << noise << t
      <<"initial"<<"noise"<<"t"
      <<PARAMETER<< delta << A << B << dispu << dispv
                 <<"delta"<<"A"<<"B"<< "u*"  << "v*";

    win.geometry(i_max,j_max)<<"Gray-Scott model"<<perturb;
    SW.geometry(SWi,SWj)<<"pattern";

    initialize();  
    d=win.disp(); 
    white=graphic_context("white");
    black=graphic_context("black");

    initial(0);
    draw(1);

    double du2,dv2,DTxx,vDTxx,uvv;
    DTxx=DT/(DX*DX);
     
/*----- main routine --------*/

    while(1){
         checkevent();
	 
         vDTxx=delta*delta*DTxx;

         for(int ts=0; ts<DISPSTEP; ts++){
            for(int j=2;j<N-1;j++){
	       du2=u[j+1]+u[j-1]-2.0*u[j];
	       dv2=v[j+1]+v[j-1]-2.0*v[j];
	       u2[j]=u[j]+DTxx*du2;
	       v2[j]=v[j]+vDTxx*dv2;
            }

            for(int j=2;j<N-1;j++){
	       uvv=u[j]*v[j]*v[j];
	       u[j]=u2[j]+DT*(-uvv+A*(1-u[j]));
	       v[j]=v2[j]+DT*(uvv-B*v[j]);
	    }
	    
            u[0]=u[N-3]; u[N]=u[3];
            u[1]=u[N-2]; u[N-1]=u[2];
            v[0]=v[N-3]; v[N]=v[3];
            v[1]=v[N-2]; v[N-1]=v[2];

            t += DT;
        }

        draw(init);
        if(init) init=0;
    }
     
    delete [] u;    delete [] u2;
}


/*************** initialize function ************/

void initial(int)
{ 
    for(int j=2; j<N-1; j++) u[j]=v[j]=0;
    init=1;  t=0;
}


void perturb(int)
{
    static int push=1;
    static int x1,x2;

    if(push){ x1=win.ev.xbutton.x; push=0; return;}
    else    { x2=win.ev.xbutton.x; push=1;}

    if(x1<x2){
      fprintf(stderr,"perturb: [%d,%d]\n",x1,x2);
      for(int j=x1; j<x2; j++) v[j]+=1.0;
    }else{
      fprintf(stderr,"perturb: [0,%d] [%d,i_max]\n",x2,x1);	
      for(int j=0; j<x2; j++) v[j]+=1.0;
      for(int j=x1; j<i_max; j++) v[j]+=1.0;
    }
	    
    u[0]=u[N-3]; u[N]=u[3];
    u[1]=u[N-2]; u[N-1]=u[2];
    v[0]=v[N-3]; v[N]=v[3];
    v[1]=v[N-2]; v[N-1]=v[2];
}


void noise(int)
{
    for(int j=2; j<N-1; j++){ 
       u[j]+=u_start*RAN;
       v[j]+=v_start*RAN;
    }
	    
    u[0]=u[N-3]; u[N]=u[3];
    u[1]=u[N-2]; u[N-1]=u[2];
    v[0]=v[N-3]; v[N]=v[3];
    v[1]=v[N-2]; v[N-1]=v[2];
}


void draw(int initial)
{
#ifndef XRSBG
    static int  sj,jpv;   
    static int *pu=new int [N];
    static int *pv=new int [N];    
    double Ampu=0.4*dispu*(double)j_max;
    double Ampv=0.4*dispv*(double)j_max;

    if(initial){ win.clear(); 
                 win.draw_line(3,0,(int)(0.5*j_max),i_max,(int)(0.5*j_max)); }

    sj--; 
    if(initial || sj<50){ SW.clear(); sj=SWj-50;}

    for(int m=0; m<N; m++){
       int j=-integer(Ampu*u[m]);
       j+=j_max/2;
       if(pu[m]!=j){
          win.erase_point(0,m,pu[m]); 
          win.draw_point(0,m,j);
          pu[m]=j;
       }
       
       j=-integer(Ampv*v[m]);        
       if(sj%5==0 && m>0) XDrawLine(d,SW,black,m-1,jpv+sj,m,j+sj);
       jpv=j;
       j+=j_max/2;
       if(pv[m]!=j){
          win.erase_point(2,m,pv[m]); 
          win.draw_point(2,m,j);
          pv[m]=j;
       }
    }
    
#endif
}








