/*---------------------------------------------------------------------------
  Kahn-Hilliard equation

     ∂t u = -△{ f(u) + △u }, f(u) = εu - u^3

---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Plane.h>
#include <XRS/Graphic.h>
using namespace XRS;

/************** 大域変数と関数宣言 ********************/

double          ep;
double          DT,DX;
double          **u;
double          L;
double          t;
int             Wimax,Wjmax,Dispstep=10;

void            initial(int);
void            boundary(void);
double          f(double u){ return (ep-u*u)*u;}

/*---------------- 以下 XWindow 用 --------------------*/

Control_Window  CW;
Graphic_Window  W,SW;
double          amp;

void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_KahnHilliard.pdf",NULL);}  }
double          linear(double x) { return x;}
double          scale(double x) { return amp*x+0.5;}

unsigned long*  color(int n)
{ //---- 濃さが連続的に変わる2種類の色を１つの配列として用意する。-----
    unsigned long *c=new unsigned long[n];
    for(int i=0; i<n; i++){
      if( i <  n/2 ) c[i]=color_pixel(0.5,1.0-2*(double)i/(double)n);
      else           c[i]=color_pixel(0.6,2*(double)i/(double)n-1.0);
    }
    return c;
}


/********** main ***********/

int main(int argc,char **argv)
{
   /*--------------- initialize ---------------------*/
   
   if(argc>1) L=atof(argv[1]);      else L=200;
   if(argc>2) DX=atof(argv[2]);     else DX=1.0;
   if(argc>3) DT=atof(argv[3]);     else DT=0.01;
   if(argc>4) ep=atof(argv[4]);     else ep=1.0;
   
   fprintf(stderr,"usage%% excitable [L] [DX] [DT] [ep]\n"
          " current systemsize: L=%g\n"
          " current parameter : ep=%g\n"
	  " space and time meshsize: DX=%g DT=%g\n"
   	  ,L,ep,DX,DT); 

   amp=0.45/sqrt(ep);
   Wimax=Wjmax=(int)(L/DX);  
   array(u,Wimax+4,Wjmax+4); 
 

   double   **u2,**u3,**f2;
   array(u2,Wimax+4,Wjmax+4); 
   array(u3,Wimax+4,Wjmax+4); 
   array(f2,Wimax+4,Wjmax+4); 

/*----- initialize ----------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;

   CW<<t<<"t"<<initial<<"initial"<<PARAMETER<< ep <<"ep"<<amp<<"amp";
   W.geometry(Wimax+4,Wjmax+4,2)<<COLOR(color(21),21)<<scale<<"Kahn-Hilliard equation eq.";
   SW.geometry(200,20,1)<<COLOR(color(21),21)<<linear<<"sample";   

   initialize();
   sample_window(SW);

/*----- main routine --------*/

   double   tx4,tx2,du4,df2;
   
   initial(1);
   
   while(1){
      tx2=DT/(DX*DX);
      tx4=DT/(DX*DX*DX*DX);
      W.draw_2D(u);

      for(int ts=0; ts<Dispstep;ts++){
        checkevent();

        for(int i=1;i<Wimax+3;i++)
          for(int j=1;j<Wjmax+3;j++){
             u2[i][j]=u[i-1][j]+u[i+1][j]+u[i][j-1]+u[i][j+1]-4.0*u[i][j];
             f2[i][j]=f(u[i][j]);
          }

        for(int i=2;i<Wimax+2;i++)
          for(int j=2;j<Wjmax+2;j++){
             du4=u2[i-1][j]+u2[i+1][j]+u2[i][j-1]+u2[i][j+1]-4.0*u2[i][j];
             df2=f2[i-1][j]+f2[i+1][j]+f2[i][j-1]+f2[i][j+1]-4.0*f2[i][j];
             u3[i][j]=u[i][j]-tx4*du4-tx2*df2;
          }

        for(int i=2;i<Wimax+2;i++)
          for(int j=2;j<Wjmax+2;j++)  u[i][j]=u3[i][j];
	
        boundary();
      }
      
      t+=DT*Dispstep;
   }
}


/**********************  初期条件 *************************/

void initial(int n)
{
    srand48(1234567);

    for(int j=2; j<Wjmax+2; j++)
        for(int i=2; i<Wimax+2; i++){ u[i][j]=drand48()-0.5;}

    boundary();
    t=0;
}


/**********************  境界条件 *************************/

void boundary(void)
{
  /*---- periodic boundary ----*/
    for(int j=0;j<Wjmax+4;j++){
        u[0][j]=u[Wimax][j]; 
        u[1][j]=u[Wimax+1][j];
        u[Wimax+2][j]=u[2][j];
        u[Wimax+3][j]=u[3][j]; 
    }

    for(int i=0;i<Wimax+4;i++){
        u[i][0]=u[i][Wjmax]; 
        u[i][1]=u[i][Wjmax+1];
        u[i][Wjmax+2]=u[i][2];
        u[i][Wjmax+3]=u[i][3]; 
    }
}









    


