/*---------------------------------------------------------------------------
  Phase Field Model of crystal growth.

   τε^2 ∂t P = ε^2 △P+P*(1-P)*(P-0.5+m(T,θ)+δ*RANSU)
   ∂t T = △T+K∂t P
   m(T,θ) = α f(θ) atan(β(1-T))/π
   (cosθ,sinθ)=∇P/|∇P|

---------------------------------------------------------------------------*/

/*********** include **************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Graphic.h>
using namespace XRS;

#define PI          M_PI
#define Random      (drand48()-0.5)
#define integer(x)  ((int)(floor(x)+floor(2*((x)-floor(x)))))


/************** 大域変数と関数宣言 ********************/

double          t;
double          **P,**T;
double          DT,DX;
double          L,alpha,beta,tau,K,e,sigma;
double          delta,r_start;
int             Wimax,Wjmax,Dispstep=10;

void            initial(int);
inline double   M(double x,double theta)
{    double co=cos(3.0*theta);
     return alpha*(1.0-sigma*co*co)*atan(beta*(1-x))/PI;
}

void            boundary(void);

/*---------------- 以下 XWindow 用 --------------------*/

Control_Window  CW;
Graphic_Window  PW,TW;
int             PWsize;
void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_PhaseField.pdf",NULL);}  }

double fT(double T){ double x=0.8*T; if(x<1){ return x;}else{ return 1.0;} }

/********** main ***********/

int main(int argc,char **argv)
{
   int ar=0;
   ar++; if(argc>ar) L=atof(argv[ar]);     else L=3;
   ar++; if(argc>ar) DT=atof(argv[ar]);    else DT=0.0001;
   ar++; if(argc>ar) DX=atof(argv[ar]);    else DX=0.03;
   ar++; if(argc>ar) K=atof(argv[ar]);     else K=1.50;      
   ar++; if(argc>ar) alpha=atof(argv[ar]); else alpha=0.9;
   ar++; if(argc>ar) beta=atof(argv[ar]);  else beta=10.0;   
   ar++; if(argc>ar) tau=atof(argv[ar]);   else tau=3.0;
   ar++; if(argc>ar) e=atof(argv[ar]);     else e=0.01;
   ar++; if(argc>ar) delta=atof(argv[ar]); else delta=0.2;
   ar++; if(argc>ar) sigma=atof(argv[ar]); else sigma=0;   

   r_start=0.15;

   fprintf(stderr,
     "usage%% CGL_1d [L] [DT] [DX] [K] [alpha] [beta] [tau] [e] [delta] [sigma]\n"
     " current systemsize: L=%g\n"
     " current parameter : K=%g alpha=%g beta=%g tau=%g e=%g delta=%g sigma=%g\n"
     " space and time meshsize: DX=%g DT=%g\n"
     ,L,K,alpha,beta,tau,e,delta,sigma,DX,DT); 

   Wimax=Wjmax=(int)(L/DX);  
   array(P,Wimax+1,Wjmax+1); 
   array(T,Wimax+1,Wjmax+1);

   double   **T2,**P2,**P3;
   array(P2,Wimax+1,Wjmax+1); array(P3,Wimax+1,Wjmax+1);
   array(T2,Wimax+1,Wjmax+1);

/*----- initialize ----------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;

   CW<< t << initial<<"t"<<"initial"<<PARAMETER
     << alpha << beta << K << tau << e << sigma
     <<"alpha"<<"beta"<<"K"<<"tau"<<"e"<<"sigma";
   
   PW.geometry(Wimax+1,Wjmax+1,2)<<"P(order parameter)";
   TW.geometry(Wimax+1,Wjmax+1,2)<<"T(temperature)"<<MONO(0.05)<<fT;

   initialize();  

   int i,j,ts;

/*----- main routine --------*/

   double   nl,dP,dT,tee,txx,txxp,theta,dPx,dPy;
   
   initial(0);   
   while(1){
      checkevent();

      tee=DT/(e*e*tau);
      txx=DT/(DX*DX);
      txxp=txx/tau;

/*----- P local map -----*/

      for(ts=0; ts<Dispstep;ts++){
        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
	     dPx=P[i+1][j]-P[i-1][j];
	     dPy=P[i][j+1]-P[i][j-1];
	     theta=atan2(dPx,dPy);
             nl=P[i][j]*(1-P[i][j])
	       *(P[i][j]-0.5+M(T[i][j],theta)+delta*Random);
             P2[i][j]=P[i][j]+tee*nl;
          }

/*----- P & T diffusion term -----*/

        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
             dP=P[i-1][j]+P[i+1][j]+P[i][j-1]+P[i][j+1]-4.0*P[i][j];
             P3[i][j]=P2[i][j]+txxp*dP;
             dT=T[i-1][j]+T[i+1][j]+T[i][j-1]+T[i][j+1]-4.0*T[i][j];
             T2[i][j]=T[i][j]+txx*dT;
          }

/*----- T local map -----*/
        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
             T[i][j]=T2[i][j]+K*(P3[i][j]-P[i][j]);
             if( T[i][j] < 0 ) T[i][j]=0.0;
             P[i][j]=P3[i][j];
          }
	
        boundary();
      }
      
      t+=DT*Dispstep;
      PW.draw_2D(P);
      TW.draw_2D(T);
   }
}


/**********************  初期条件 *************************/

void initial(int)
{
    for(int j=0; j<Wjmax+1; j++)
        for(int i=0; i<Wimax+1; i++){ P[i][j] = 0; T[i][j] = 0; }

    for(int j=1; j<Wjmax; j++)
        for(int i=1; i<Wimax; i++)
            if(((i-1)*(i-1)+(j-1)*(j-1))<(r_start/DX*r_start/DX)){
                T[i][j] = 1.0; P[i][j] = 1.0;
            }

    boundary();
    t=0;
}


/**********************  境界条件 *************************/

void boundary(void)
{
    for(int j=1;j<Wjmax;j++){
        T[0][j]=T[1][j]; P[0][j]=P[1][j];
        T[Wimax][j]=0;   P[Wimax][j]=0;
    }
    
    for(int i=1;i<Wimax;i++){
        T[i][0]=T[i][1]; P[i][0]=P[i][1];
        T[i][Wjmax]=0;   P[i][Wjmax]=0;
    }
}







    


