/*=================================================================
    R\"ossler 方程式

    dxdt=-y-z;
    dydt=x+a*y;
    dzdt=b-c*z+x*z;
=================================================================*/

/*************************************************************
    インクルードと大域変数
*************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include "include/ode.h"          // 常微分方程式を解くサブルーチン
#include "include/project.h"      // 3次元の点を平面に射影するサブルーチン

double a=0.2,b=0.2,c=5.8;
double *xp,**yp,*x,*y,dxsav,xx=6,yy=0.1,zz=0.0;
int    kmax,kount;


/*************************************************************
    グラフィック用のインクルードと変数、関数の宣言
*************************************************************/

#include <XRS/Control.h>
#include <XRS/Plane.h>
using namespace XRS;

#define WinI(x)  (int)((double)width*(0.5+Amp*(x)))
#define WinJ(x)  (int)((double)height*(0.5-Amp*(x)))
#define WinX(i)  (double)(((double)i/width-0.5)/Amp)
#define WinY(j)  (double)((0.5-(double)j/height)/Amp)

double         Amp=0.02;
int            height=500,width=500,marker=1,nmax=2000,slow=200;
axis           axy;    
double         theta=-0.2,phi=0.1;
Plane_Window   Win;
Control_Window CW,SubCW(Win);
void           draw();
void           startpoint(int);
void           explain(int)
{  if(!fork()){ execlp("evince","evince","README_Rossler.pdf",NULL);}  }
void           save(int);

/************************************************************
    方程式
*************************************************************/

void derivs(double x,double y[],double dydx[]){
    double X=y[1],Y=y[2],Z=y[3];
    dydx[1]= -Y-Z;
    dydx[2]= X+a*Y;
    dydx[3]= b-c*Z+X*Z;
}


/************************************************************
    メインルーチン
*************************************************************/

int main(int argc,char *argv[])
{
    int nvar=3;  // 方程式の変数の数
    int nok,nbad;
    double x1,x2,eps,h1,hmin,hmax;
    double *ystart=vector(1,nvar);

    /*----- ウインドウの初期設定 ---------*/

    CW.realtime=0;
    f_dialog d(CW,explain);
    d.resize(2*d.width(),d.height());
    d.string(20);
    d.title(" readme (Click here)");
    CW.realtime=1;
    
    CW<<PARAMETER<<"a"<<a<<b<<"b"<<c<<"c"<<theta<<"theta"<<phi<<"phi"
                 <<"Amp"<<Amp
      <<"nmax"<<nmax<<"slow"<<slow<<"save"<<save;
    
    Win.geometry(width,height)
	<<"Roessler equation (crick this window! )"<<startpoint;
    SubCW<<"x0"<<xx<<"y0"<<yy<<PARAMETER<<"z0"<<zz;
    
    initialize();

    /*----- パラメータが変更されるたびに解き直す。 ---------*/

    while(1){
	
      ystart[1]=xx;  // 初期条件
      ystart[2]=yy;
      ystart[3]=zz;

      x1=0;          // 初期時刻
      x2=1000;       // 終了時刻
      eps=0.00001;   // 精度
      h1=0.002;      // 初期の刻み幅
      hmin=0.0;      // 刻み幅の最小値
      hmax=0.002;    // 刻み幅の最大値
      kmax=100000;   // 配列の大きさ
      dxsav=0.01;    
	
      xp=vector(1,kmax);
      yp=matrix(1,nvar,1,kmax);

      // 適応刻み幅制御つきのルンゲクッタ法
      odeint(ystart,nvar,x1,x2,eps,h1,hmin,hmax,&nok,&nbad,derivs,rkqs);

      // 得られた点を平面に射影する。
      x=new double [kount-1];
      y=new double [kount-1];
      axy=project(theta,phi,kount-1,
                  &yp[1][1],&yp[2][1],&yp[3][1],x,y);

      // 描画
      draw();

      // イベント発生を検出するまで待つ。
      Win.chev=CW.chev=0;
      while(!CW.chev && !Win.chev ) checkevent();
      CW.draw();
   
      delete [] x;
      delete [] y;
    }
}


void save(int)
{
   for(int n=1; n<kount-1; n++){ 
      printf("%g %g %g\n",yp[1][n],yp[2][n],yp[3][n]);
   }	   
}


/************************************************************
     描画
*************************************************************/

void draw_mark(int,int,int,int);
void erase_mark(int,int,int,int);
#include <time.h>
struct timespec treq;
long   usec=1E3;
	
void draw()
// 結果の表示 
{
   int    i0,j0,i,j;

   Win.clear(0); 
   Win.clear(1);
   Win.clear(2); 

   for(int k=0; k<3; k++){
     Win.draw_line(2,WinI(axy.x[0][k]),WinJ(axy.y[0][k]),
                     WinI(axy.x[1][k]),WinJ(axy.y[1][k])); 
   }

   i0=WinI(x[0]); j0=WinJ(y[0]);
   for(int n=1; n<kount-1; n++){ 
       if(marker && n>nmax) break;       
       i=WinI(x[n]); j=WinJ(y[n]);
       if(marker){ // トレーサー付き
  	   treq.tv_nsec=usec*slow;  nanosleep(&treq,NULL);  // 実行を遅らせる
	   erase_mark(0,i0,j0,5);
	   draw_mark(0,i,j,5);
       }	   
       Win.draw_line(1,i0,j0,i,j);    // 軌跡の描画
       i0=i; j0=j;
   }
}    


void draw_mark(int c,int x,int y,int r)
// ウィンドウ Win に色 c で 中心(i,j) 半径 r の塗りつぶされた円を書く。
{
    for(int i=-r; i<r+1; i++)
	for(int j=-r; j<r+1; j++)
	    if(i*i+j*j<=r*r) Win.draw_point(c,i+x,j+y);
}


void erase_mark(int c,int x,int y,int r)
// ウィンドウ Win に色 c の 中心(i,j) 半径 r の塗りつぶされた円を消す。
{    
    for(int i=-r; i<r+1; i++)
	for(int j=-r; j<r+1; j++)
	    if(i*i+j*j<=r*r) Win.erase_point(c,i+x,j+y);
}    


void   startpoint(int n)
// マウスが押された位置の座標を初期点とする。また、右ボタンが押されたら、
// 表示モードを変更する。
{
    projected_point P;
    double wx,wy;
    wx=WinX(Win.ev.xbutton.x);
    wy=WinY(Win.ev.xbutton.y);

    P=project_xytoXY(theta,phi,zz,wx,wy);
    xx=P.X;
    yy=P.Y;

    if(n==3) marker=!marker; 
}    
